/*****************************************************************************************************************************
--
--    File Name    : Display_Enhancements.v 

--    Description  : This module does brightness, contrast, saturation and hue adjustment to YCbCr
--                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module DisplayEnhancements #(parameter g_YCbCr_DATA_BIT_WIDTH = 8,
                              parameter g_B_C_SAT_CONSTANT_WIDTH = 8,
                              parameter g_HUE_CONSTANT_WIDTH = 9,
                              parameter g_SIN_COS_DWIDTH = 10,
                              parameter g_SIN_COS_MEM_DEPTH = 180)
                            (input wire VIDEO_CLOCK_I,
                             input wire mem_write_clock,
                             input wire RESET_N_I,
                             input signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] brightness_i,
                             input signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] contrast_i,
                             input signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] saturation_i,
                             input signed[(g_HUE_CONSTANT_WIDTH - 1) : 0] hue_i,
                             input wire sin_write_en_i,
                             input wire cos_write_en_i,
                             input wire[7 : 0] sin_write_addr_i,
                             input wire[7 : 0] cos_write_addr_i,
                             input wire[(g_SIN_COS_DWIDTH - 1) : 0] sin_value_i,
                             input wire[(g_SIN_COS_DWIDTH - 1) : 0] cos_value_i,
                             input wire[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Y_i,
                             input wire[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cb_i,
                             input wire[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cr_i,
                             input wire data_enable_i,
                             input wire horz_sync_i,
                             input wire vert_sync_i,
                             input wire hactive_i,
                             input wire vactive_i,
                             output reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Y_out_o,
                             output reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cb_out_o,
                             output reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] Cr_out_o,
                             output reg data_enable_o,
                             output reg horz_sync_o,
                             output reg vert_sync_o,
                             output reg hactive_o,
                             output reg vactive_o);
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] O;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] I;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] l;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] o;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] i;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] OI;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] II;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] lI;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 1) : 0] oI;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 2) : 0] iI;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 3) : 0] Ol;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 4) : 0] Il;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 5) : 0] ll;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] ol;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] il;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] O0;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] I0;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] l0;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 1) : 0] o0;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 2) : 0] i0;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 3) : 0] O1;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 4) : 0] I1;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 5) : 0] l1;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] o1;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] i1;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] Oo;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] Io;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] lo;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] oo;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] io;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] Oi;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] Ii;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] li;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] oi;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] ii;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] OOI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] IOI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] lOI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] oOI;
  reg[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] iOI;
  reg[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] OII;
  reg[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] III;
  reg[(g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] lII;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] oII;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] iII;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] OlI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] IlI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] llI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] olI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] ilI;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] O0I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] I0I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] l0I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] o0I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] i0I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] O1I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] I1I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] l1I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] o1I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] i1I;
  reg signed[(g_B_C_SAT_CONSTANT_WIDTH) : 0] OoI;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] IoI;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] loI;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] ooI;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] ioI;
  reg[(g_YCbCr_DATA_BIT_WIDTH - 1) : 0] OiI;
  reg[(g_YCbCr_DATA_BIT_WIDTH) : 0] IiI;
  reg[(g_YCbCr_DATA_BIT_WIDTH) : 0] liI;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 1) : 0] oiI;
  reg[(g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH - 1) : 0] iiI;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 1) : 0] OOl;
  reg signed[(g_HUE_CONSTANT_WIDTH - 1) : 0] IOl;
  reg signed[(g_HUE_CONSTANT_WIDTH - 1) : 0] lOl;
  reg signed[(g_SIN_COS_DWIDTH - 1) : 0] oOl;
  reg signed[(g_SIN_COS_DWIDTH - 1) : 0] iOl;
  reg signed[(g_SIN_COS_DWIDTH - 1) : 0] OIl;
  reg signed[(g_SIN_COS_DWIDTH - 1) : 0] IIl;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 1) : 0] lIl;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 1) : 0] oIl;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 1) : 0] iIl;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 1) : 0] Oll;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 1) : 0] Ill;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 1) : 0] lll;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 2) : 0] oll;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH + 2) : 0] ill;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] O0l;
  reg signed[(g_YCbCr_DATA_BIT_WIDTH) : 0] I0l;
  reg l0l;
  reg o0l;
  reg i0l;
  reg O1l;
  reg I1l;
  reg l1l;
  reg o1l;
  reg i1l;
  reg Ool;
  reg Iol;
  reg lol;
  reg ool;
  reg iol;
  reg Oil;
  reg Iil;
  reg lil;
  reg oil;
  reg iil;
  reg OO0;
  reg IO0;
  reg lO0;
  reg oO0;
  reg iO0;
  reg OI0;
  reg II0;
  reg lI0;
  reg oI0;
  reg iI0;
  reg Ol0;
  reg Il0;
  reg ll0;
  reg ol0;
  reg il0;
  reg O00;
  reg I00;
  reg l00;
  reg o00;
  reg i00;
  reg O10;
  reg I10;
  reg l10;
  reg o10;
  reg i10;
  reg Oo0;
  reg Io0;
  reg lo0;
  reg oo0;
  reg io0;
  reg Oi0;
  reg Ii0;
  reg li0;
  reg oi0;
  reg ii0;
  reg OO1;
  reg IO1;
  reg lO1;
  reg oO1;
  reg iO1;
  reg OI1;
  reg II1;
  reg lI1;
  reg oI1;
  reg iI1;
  reg Ol1;
  reg Il1;
  wire signed[(g_SIN_COS_DWIDTH - 1) : 0] ll1;
  wire signed[(g_SIN_COS_DWIDTH - 1) : 0] ol1;
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          O <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          o <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          ol <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          l0l <= 1'b0;
          Oil <= 1'b0;
          oI0 <= 1'b0;
          I10 <= 1'b0;
          ii0 <= 1'b0;
          o1 <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          OOI <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          oII <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          I0I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          IOl <= { g_HUE_CONSTANT_WIDTH { 1'b0 } };
        end
      else
        begin
          O <= Y_i;
          o <= { 1'b0, Cb_i };
          ol <= { 1'b0, Cr_i };
          l0l <= horz_sync_i;
          Oil <= vert_sync_i;
          oI0 <= data_enable_i;
          I10 <= hactive_i;
          ii0 <= vactive_i;
          o1 <= brightness_i;
          OOI <= contrast_i;
          oII <= { contrast_i[(g_B_C_SAT_CONSTANT_WIDTH - 1)], contrast_i };
          I0I <= { saturation_i[(g_B_C_SAT_CONSTANT_WIDTH - 1)], saturation_i };
          IOl <= hue_i;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          lOl <= { g_HUE_CONSTANT_WIDTH { 1'b0 } };
        end
      else
        begin
          if (IOl[(g_HUE_CONSTANT_WIDTH - 1)] == 1'b1)
            begin
              lOl <= (~IOl) + 1'b1;
            end
          else
            begin
              lOl <= IOl;
            end
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          I <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          i <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          il <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          o0l <= 1'b0;
          Iil <= 1'b0;
          iI0 <= 1'b0;
          l10 <= 1'b0;
          OO1 <= 1'b0;
          i1 <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          IOI <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          iII <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          l0I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          I <= O;
          i <= o;
          il <= ol;
          o0l <= l0l;
          Iil <= Oil;
          iI0 <= oI0;
          l10 <= I10;
          OO1 <= ii0;
          i1 <= o1;
          IOI <= OOI;
          iII <= oII;
          l0I <= I0I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          oOl <= { g_SIN_COS_DWIDTH { 1'b0 } };
          iOl <= { g_SIN_COS_DWIDTH { 1'b0 } };
        end
      else
        begin
          if (IOl[(g_HUE_CONSTANT_WIDTH - 1)] == 1'b1)
            begin
              oOl <= (~ll1) + 1'b1;
              iOl <= ol1;
            end
          else
            begin
              oOl <= ll1;
              iOl <= ol1;
            end
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          l <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          OI <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          O0 <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          i0l <= 1'b0;
          lil <= 1'b0;
          Ol0 <= 1'b0;
          o10 <= 1'b0;
          IO1 <= 1'b0;
          Oo <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          lOI <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          OlI <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          o0I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          l <= I;
          OI <= i;
          O0 <= il;
          i0l <= o0l;
          lil <= Iil;
          Ol0 <= iI0;
          o10 <= l10;
          IO1 <= OO1;
          Oo <= i1;
          lOI <= IOI;
          OlI <= iII;
          o0I <= l0I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          IoI <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          II <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          I0 <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          O1l <= 1'b0;
          oil <= 1'b0;
          Il0 <= 1'b0;
          i10 <= 1'b0;
          lO1 <= 1'b0;
          Io <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          oOI <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          IlI <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          i0I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          OIl <= { g_SIN_COS_DWIDTH { 1'b0 } };
          IIl <= { g_SIN_COS_DWIDTH { 1'b0 } };
        end
      else
        begin
          IoI <= l - 8'd16;
          II <= OI - $signed(9'd128);
          I0 <= O0 - $signed(9'd128);
          O1l <= i0l;
          oil <= lil;
          Il0 <= Ol0;
          i10 <= o10;
          lO1 <= IO1;
          Io <= Oo;
          oOI <= lOI + $signed(8'd128);
          IlI <= OlI + $signed(9'd128);
          i0I <= o0I + $signed(9'd128);
          OIl <= oOl;
          IIl <= iOl;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          loI <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          lIl <= 0;
          oIl <= 0;
          iIl <= 0;
          Oll <= 0;
          I1l <= 1'b0;
          iil <= 1'b0;
          ll0 <= 1'b0;
          Oo0 <= 1'b0;
          oO1 <= 1'b0;
          lo <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          iOI <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          llI <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          O1I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          loI <= IoI;
          lIl <= (II * IIl);
          oIl <= (I0 * OIl);
          iIl <= (I0 * IIl);
          Oll <= (II * OIl);
          I1l <= O1l;
          iil <= oil;
          ll0 <= Il0;
          Oo0 <= i10;
          oO1 <= lO1;
          lo <= Io;
          iOI <= $unsigned(oOI);
          llI <= IlI;
          O1I <= i0I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          ooI <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          Ill <= { (g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 2) { 1'b0 } };
          lll <= { (g_YCbCr_DATA_BIT_WIDTH + g_SIN_COS_DWIDTH + 2) { 1'b0 } };
          l1l <= 1'b0;
          OO0 <= 1'b0;
          ol0 <= 1'b0;
          Io0 <= 1'b0;
          iO1 <= 1'b0;
          oo <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          OII <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          olI <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          I1I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          ooI <= loI;
          Ill <= lIl + oIl;
          lll <= iIl - Oll;
          l1l <= I1l;
          OO0 <= iil;
          ol0 <= ll0;
          Io0 <= Oo0;
          iO1 <= oO1;
          oo <= lo;
          OII <= iOI;
          olI <= llI;
          I1I <= O1I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          ioI <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          oll <= { (g_YCbCr_DATA_BIT_WIDTH + 3) { 1'b0 } };
          ill <= { (g_YCbCr_DATA_BIT_WIDTH + 3) { 1'b0 } };
          o1l <= 1'b0;
          IO0 <= 1'b0;
          il0 <= 1'b0;
          lo0 <= 1'b0;
          OI1 <= 1'b0;
          io <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          III <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          ilI <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          l1I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          ioI <= ooI;
          oll <= Ill >> 9;
          ill <= lll >> 9;
          o1l <= l1l;
          IO0 <= OO0;
          il0 <= ol0;
          lo0 <= Io0;
          OI1 <= iO1;
          io <= oo;
          III <= OII;
          ilI <= olI;
          l1I <= I1I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          OiI <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          O0l <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          I0l <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          iol <= 1'b0;
          lI0 <= 1'b0;
          O10 <= 1'b0;
          oi0 <= 1'b0;
          Il1 <= 1'b0;
          ii <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          lII <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          O0I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
          OoI <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          OiI <= ioI;
          O0l <= { oll[10], oll[7 : 0] };
          I0l <= { ill[10], ill[7 : 0] };
          iol <= o1l;
          lI0 <= IO0;
          O10 <= il0;
          oi0 <= lo0;
          Il1 <= OI1;
          ii <= io;
          lII <= III;
          O0I <= ilI;
          OoI <= l1I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          iiI <= { (g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH) { 1'b0 } };
          oI <= { (g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 2) { 1'b0 } };
          o0 <= { (g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 2) { 1'b0 } };
          i1l <= 1'b0;
          lO0 <= 1'b0;
          O00 <= 1'b0;
          oo0 <= 1'b0;
          II1 <= 1'b0;
          Oi <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          o1I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          iiI <= OiI * lII;
          oI <= O0l * O0I;
          o0 <= I0l * O0I;
          i1l <= iol;
          lO0 <= lI0;
          O00 <= O10;
          oo0 <= oi0;
          II1 <= Il1;
          Oi <= ii;
          o1I <= OoI;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          IiI <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          iI <= { (g_YCbCr_DATA_BIT_WIDTH + 3) { 1'b0 } };
          i0 <= { (g_YCbCr_DATA_BIT_WIDTH + 3) { 1'b0 } };
          Ool <= 1'b0;
          oO0 <= 1'b0;
          I00 <= 1'b0;
          io0 <= 1'b0;
          lI1 <= 1'b0;
          Ii <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
          i1I <= { (g_B_C_SAT_CONSTANT_WIDTH + 1) { 1'b0 } };
        end
      else
        begin
          IiI <= (iiI >> 7);
          iI <= (oI >> 7);
          i0 <= (o0 >> 7);
          Ool <= i1l;
          oO0 <= lO0;
          I00 <= O00;
          io0 <= oo0;
          lI1 <= II1;
          Ii <= Oi;
          i1I <= o1I;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          liI <= { (g_YCbCr_DATA_BIT_WIDTH + 1) { 1'b0 } };
          Ol <= { (g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 4) { 1'b0 } };
          O1 <= { (g_YCbCr_DATA_BIT_WIDTH + g_B_C_SAT_CONSTANT_WIDTH + 4) { 1'b0 } };
          Iol <= 1'b0;
          iO0 <= 1'b0;
          l00 <= 1'b0;
          Oi0 <= 1'b0;
          oI1 <= 1'b0;
          li <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
        end
      else
        begin
          liI <= IiI;
          Ol <= iI * i1I;
          O1 <= i0 * i1I;
          Iol <= Ool;
          iO0 <= oO0;
          l00 <= I00;
          Oi0 <= io0;
          oI1 <= lI1;
          li <= Ii;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          oiI <= { ((g_YCbCr_DATA_BIT_WIDTH + 1) + 1) { 1'b0 } };
          Il <= { ((g_YCbCr_DATA_BIT_WIDTH + 4) + 1) { 1'b0 } };
          I1 <= { ((g_YCbCr_DATA_BIT_WIDTH + 4) + 1) { 1'b0 } };
          lol <= 1'b0;
          OI0 <= 1'b0;
          o00 <= 1'b0;
          Ii0 <= 1'b0;
          iI1 <= 1'b0;
          oi <= { g_B_C_SAT_CONSTANT_WIDTH { 1'b0 } };
        end
      else
        begin
          oiI <= { 1'b0, liI };
          Il <= (Ol >> 7);
          I1 <= (O1 >> 7);
          lol <= Iol;
          OI0 <= iO0;
          o00 <= l00;
          Ii0 <= Oi0;
          iI1 <= oI1;
          oi <= li;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          OOl <= { ((g_YCbCr_DATA_BIT_WIDTH + 1) + 1) { 1'b0 } };
          ll <= { ((g_YCbCr_DATA_BIT_WIDTH + 5) + 1) { 1'b0 } };
          l1 <= { ((g_YCbCr_DATA_BIT_WIDTH + 5) + 1) { 1'b0 } };
          ool <= 1'b0;
          II0 <= 1'b0;
          i00 <= 1'b0;
          li0 <= 1'b0;
          Ol1 <= 1'b0;
        end
      else
        begin
          OOl <= oiI + oi + $signed(9'd16);
          ll <= Il + $signed(13'd128);
          l1 <= I1 + $signed(13'd128);
          ool <= lol;
          II0 <= OI0;
          i00 <= o00;
          li0 <= Ii0;
          Ol1 <= iI1;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I or
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          Y_out_o <= 8'd16;
        end
      else
        begin
          if (OOl > $signed({ { ((g_YCbCr_DATA_BIT_WIDTH + 2) - 8) { 1'b0 } }, 8'd235 }))
            Y_out_o <= 8'd235;
          else
            if (OOl < $signed({ { ((g_YCbCr_DATA_BIT_WIDTH + 2) - 8) { 1'b0 } }, 8'd16 }))
              Y_out_o <= 8'd16;
            else
              Y_out_o <= OOl[7 : 0];
        end
    end
  always
    @(posedge VIDEO_CLOCK_I or
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          Cb_out_o <= 8'd128;
        end
      else
        begin
          if (ll > $signed({ { ((g_YCbCr_DATA_BIT_WIDTH + 6) - 8) { 1'b0 } }, 8'd240 }))
            Cb_out_o <= 8'd240;
          else
            if (ll < $signed({ { ((g_YCbCr_DATA_BIT_WIDTH + 6) - 8) { 1'b0 } }, 8'd16 }))
              Cb_out_o <= 8'd16;
            else
              Cb_out_o <= ll[7 : 0];
        end
    end
  always
    @(posedge VIDEO_CLOCK_I or
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          Cr_out_o <= 8'd128;
        end
      else
        begin
          if (l1 > $signed({ { ((g_YCbCr_DATA_BIT_WIDTH + 6) - 8) { 1'b0 } }, 8'd240 }))
            Cr_out_o <= 8'd240;
          else
            if (l1 < $signed({ { ((g_YCbCr_DATA_BIT_WIDTH + 6) - 8) { 1'b0 } }, 8'd16 }))
              Cr_out_o <= 8'd16;
            else
              Cr_out_o <= l1[7 : 0];
        end
    end
  always
    @(posedge VIDEO_CLOCK_I or
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          horz_sync_o <= 1'b0;
          vert_sync_o <= 1'b0;
          data_enable_o <= 1'b0;
          hactive_o <= 1'b0;
          vactive_o <= 1'b0;
        end
      else
        begin
          horz_sync_o <= ool;
          vert_sync_o <= II0;
          data_enable_o <= i00;
          hactive_o <= li0;
          vactive_o <= Ol1;
        end
    end
  sin_mem #(.g_SIN_COS_DWIDTH(g_SIN_COS_DWIDTH), .g_HUE_CONSTANT_WIDTH(g_HUE_CONSTANT_WIDTH), .g_SIN_COS_MEM_DEPTH(g_SIN_COS_MEM_DEPTH)) il1(.rclock(VIDEO_CLOCK_I),
                                                                                                                                             .wclock(mem_write_clock),
                                                                                                                                             .nreset(RESET_N_I),
                                                                                                                                             .write_en_i(sin_write_en_i),
                                                                                                                                             .write_addr_i(sin_write_addr_i),
                                                                                                                                             .sin_value_i(sin_value_i),
                                                                                                                                             .degree_i(lOl),
                                                                                                                                             .sin_value_o(ll1));
  cos_mem #(.g_SIN_COS_DWIDTH(g_SIN_COS_DWIDTH), .g_HUE_CONSTANT_WIDTH(g_HUE_CONSTANT_WIDTH), .g_SIN_COS_MEM_DEPTH(g_SIN_COS_MEM_DEPTH)) O01(.rclock(VIDEO_CLOCK_I),
                                                                                                                                             .wclock(mem_write_clock),
                                                                                                                                             .nreset(RESET_N_I),
                                                                                                                                             .write_en_i(cos_write_en_i),
                                                                                                                                             .write_addr_i(cos_write_addr_i),
                                                                                                                                             .cos_value_i(cos_value_i),
                                                                                                                                             .degree_i(lOl),
                                                                                                                                             .cos_value_o(ol1));
endmodule
