/*****************************************************************************************************************************
--
--    File Name    : cos_mem.v 

--    Description  : This module implements cos memory
--                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module cos_mem #(parameter g_SIN_COS_DWIDTH = 10,
                 parameter g_HUE_CONSTANT_WIDTH = 8,
                 parameter g_SIN_COS_MEM_DEPTH = 180)
               (input wire wclock,
                input wire rclock,
                input wire nreset,
                input wire write_en_i,
                input wire[7 : 0] write_addr_i,
                input wire[(g_SIN_COS_DWIDTH - 1) : 0] cos_value_i,
                input wire[(g_HUE_CONSTANT_WIDTH - 1) : 0] degree_i,
                output wire[(g_SIN_COS_DWIDTH - 1) : 0] cos_value_o);
  reg[(g_SIN_COS_DWIDTH - 1) : 0] I01;
  reg[(g_SIN_COS_DWIDTH - 1) : 0] l01[g_SIN_COS_MEM_DEPTH : 0];
  always
    @(posedge wclock)
    begin
      if (write_en_i)
        begin
          l01[write_addr_i] <= cos_value_i;
        end
    end
  always
    @(posedge rclock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          I01 <= { g_SIN_COS_DWIDTH { 1'b0 } };
        end
      else
        begin
          I01 <= l01[degree_i];
        end
    end
  assign cos_value_o = I01;
endmodule
