/*****************************************************************************************************************************
--
--    File Name    : rd_master_req_latch.v

--    Description  : This module latches the external master read request to be given as input to AXI_arbiter module
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2013 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module rd_master_req_latch(input wire clock,
                           input wire nreset,
                           input wire rd_req_1,
                           input wire rd_done_1,
                           output reg rd_req_arb_1,
                           input wire rd_req_2,
                           input wire rd_done_2,
                           output reg rd_req_arb_2,
                           input wire rd_req_3,
                           input wire rd_done_3,
                           output reg rd_req_arb_3,
                           input wire rd_req_4,
                           input wire rd_done_4,
                           output reg rd_req_arb_4);
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          rd_req_arb_1 <= 1'b0;
        end
      else
        begin
          if (rd_done_1 && !rd_req_1)
            begin
              rd_req_arb_1 <= 1'b0;
            end
          else
            if (rd_req_1)
              begin
                rd_req_arb_1 <= 1'b1;
              end
            else
              begin
                rd_req_arb_1 <= rd_req_arb_1;
              end
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          rd_req_arb_2 <= 1'b0;
        end
      else
        begin
          if (rd_done_2 && !rd_req_2)
            begin
              rd_req_arb_2 <= 1'b0;
            end
          else
            if (rd_req_2)
              begin
                rd_req_arb_2 <= 1'b1;
              end
            else
              begin
                rd_req_arb_2 <= rd_req_arb_2;
              end
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          rd_req_arb_3 <= 1'b0;
        end
      else
        begin
          if (rd_done_3 && !rd_req_3)
            begin
              rd_req_arb_3 <= 1'b0;
            end
          else
            if (rd_req_3)
              begin
                rd_req_arb_3 <= 1'b1;
              end
            else
              begin
                rd_req_arb_3 <= rd_req_arb_3;
              end
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          rd_req_arb_4 <= 1'b0;
        end
      else
        begin
          if (rd_done_4 && !rd_req_4)
            begin
              rd_req_arb_4 <= 1'b0;
            end
          else
            if (rd_req_4)
              begin
                rd_req_arb_4 <= 1'b1;
              end
            else
              begin
                rd_req_arb_4 <= rd_req_arb_4;
              end
        end
    end
endmodule
